@@ -11,7 +11,7 @@ from enum import Enum
 from itertools import islice
 from queue import Queue
 from time import sleep, time
-from typing import Any, Callable, Dict, Generator, Iterable, Iterator, List, Optional, Tuple, Union
+from typing import Any, Callable, Dict, Generator, Iterable, Iterator, List, Mapping, Optional, Tuple, Union
 from uuid import uuid4
 
 import grpc
@@ -229,11 +229,11 @@ class NumerousClient:
                                                                   instance_id=self._instance_id,
                                                                   token_callback=self._get_current_token,
                                                                   refresh_token=self._refresh_token)
-        self.stub = spm_pb2_grpc.SPMStub(self.channel)
+        self._base_client = spm_pb2_grpc.SPMStub(self.channel)
         self._token_manager = spm_pb2_grpc.TokenManagerStub(self.channel)
         self._health = health_pb2_grpc.HealthStub(self.channel)
         self._job_manager = spm_pb2_grpc.JobManagerStub(self.channel)
-        self._base_client = self.stub
+        self._file_manager = spm_pb2_grpc.FileManagerStub(self.channel)
 
         self._cancel_streams: List[Callable[[], None]] = []
 
@@ -359,7 +359,8 @@ class NumerousClient:
                 self.hibernate(message='Hibernating')
                 break
 
-    def get_scenario_document(self, scenario=None, project=None):
+    def get_scenario_document(self, scenario=None, project=None) -> Tuple[Dict[str, Any],
+                                                                          Iterable[spm_pb2.FileSignedUrl]]:
         if scenario is None:
             scenario = self._scenario
         if project is None:
@@ -548,6 +549,43 @@ class NumerousClient:
         requests.post(upload_url.url, headers={"Content-Type": "multipart/related"},
                       params={"name": file, "mimeType": content_type}, data=open(local_file, 'rb'))
 
+    def upload_file_path(self, remote_path: str, local_path: str):
+        upload_urls = self._file_manager.GetUploadUrls(spm_pb2.FilePaths(project_id=self._project, paths=[remote_path]))
+
+        # Validate url
+        url = None
+        for upload_url in upload_urls.urls:
+            if upload_url.path == remote_path:
+                url = upload_url.url
+
+        if not url:
+            raise RuntimeError(f'Could not upload to the remote path {remote_path}')
+
+        with open(local_path, 'rb') as upload_file:
+            requests.post(url, headers={"Content-Type": "multipart/related"}, data=upload_file)
+
+    def download_file_path(self, remote_path: str, local_path: str, overwrite: bool = False) -> None:
+        self.download_file_paths({remote_path: local_path}, overwrite)
+
+    def download_file_paths(self, path_mapping: Mapping[str, str], overwrite: bool = False) -> None:
+        download_urls = self._file_manager.GetDownloadUrls(spm_pb2.FilePaths(project_id=self._project,
+                                                                             paths=list(path_mapping)))
+
+        for local_path in path_mapping.values():
+            if not overwrite and os.path.exists(local_path):
+                raise OSError(f'Cannot download onto existing file {local_path}')
+
+        for download_url in download_urls.urls:
+            if not download_url.url:
+                raise RuntimeError(f'Could not download file at {download_url.path}')
+
+        for download_url in download_urls.urls:
+            url = download_url.url
+            local_path = path_mapping[download_url.path]
+            with requests.get(url, allow_redirects=True, stream=True) as response, open(local_path, 'wb') as file:
+                for chunk in response.iter_content(chunk_size=2**16):
+                    file.write(chunk)
+
     def get_timeseries_stats(self, project=None, scenario=None, execution=None, tag='_index'):
         if project is None:
             project = self._project
